/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Glite Types
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SrmExceptions.h,v 1.3 2006/11/28 12:01:16 badino Exp $
 *  Release: $Name: glite-data-srm-util-cpp_R_1_0_1_4 $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_GLITE_SRM_EXCEPTIONS_H_
#define GLITE_DATA_SRM_UTIL_GLITE_SRM_EXCEPTIONS_H_

#include <vector>

#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/srm/util/types.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {

/*
 * define Exceptions
 */
typedef glite::data::agents::LogicError     LogicError;
typedef glite::data::agents::RuntimeError   RuntimeError;
typedef glite::data::agents::NullPointerException     NullPointerException;
typedef glite::data::agents::InvalidArgumentException InvalidArgumentException;

/**
 * Exception Class for reportinig Srm Call failures
 *
 * A SrmError is used to report errors calling the SRM that are not directly
 * related to the request. Examples are: Internal Server Error, Not Supported,
 * Invalid Request Ids etc. On the other end, error that are related to the 
 * evolution of the request should be reported in the request status
 */
struct SrmError : public RuntimeError{
    /**
     * Constructor
     */
    SrmError(SrmStatus::Code code, const std::string& reason);
    /**
     * Constructor
     */
    SrmError(const SrmStatus& status);
    /**
     * Destructor
     */
    virtual ~SrmError() throw(){}
        
    /**
     * Return Status
     */
    const SrmStatus& status() const throw () {return m_status;}
    
protected:
    
    /**
     * Create Error Message
     */
    static std::string getErrorMessage(SrmStatus::Code code, const std::string& reason) /*throw()*/;
private:

    /**
     * The SRM Status
     */
    SrmStatus m_status;
};

/**
 * Exception Class for reportinig inconsistencies in the Srm result
 *
 * A SrmBadResponse is used to report inconsistencies in the result returned by 
 * the SRM. In theory this erxceptuon shoudl never be thrown.
 */
struct SrmBadResponse : public RuntimeError{
    /**
     * Constructor
     */
    SrmBadResponse(const std::string& reason):RuntimeError(reason){}
    /**
     * Destructor
     */
    virtual ~SrmBadResponse() throw(){}
};

/**
 * Exception Class for reportinig failures in remote communication with the SRM
 *
 * A RemoteError is used to report error during the RPC, for example
 * host not reachable or unkwon , timeoyt on HTTP snd or recv etc
 */
struct RemoteError : public RuntimeError{
    /**
     * Constructor
     */
    RemoteError(const std::string& reason):RuntimeError(reason){}
    /**
     * Destructor
     */
    virtual ~RemoteError() throw(){}
};

/**
 * Class for reportinig secuity errors at the connection level. 
 * In case the request failed for authn/authz errors on teh server, this should 
 * be reported as the request level using the SRM_AUTHENTICATION_FAILURE or  
 * SRM_AUTHORIZATION_FAILURE codes
 */
struct SecurityError : public RuntimeError{
    /**
     * Constructor
     */
    SecurityError(const std::string& reason):RuntimeError(reason){}
    /**
     * Destructor
     */
    virtual ~SecurityError() throw(){}
};

} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_GLITE_SRM_EXCEPTIONS_H_
